open Risc

let rec from_to i j =
  if i > j then []
  else 
    i::(from_to (i+1) j)

let registers () =
  let rc_min = 1 and rc_max = 29 in
  let all_free_registers = List.map (function i -> R(i)) (from_to rc_min rc_max) in
  let free = ref all_free_registers in
  let get_register (R(n) as reg) = 
    if List.mem reg !free then
      begin
	free := List.filter (fun reg' -> reg <> reg') !free
      end
    else
      Report.fail ("register "^(string_of_register reg)^" already used (or invalid)")
  in
  let get_free_register () =
    match !free with
	[] -> Report.fail ("no more free registers")
      | reg::regs ->
	  free := regs;
	  reg
  in
  let free_register reg =
    if (List.mem reg all_free_registers) then
      if List.mem reg !free then
	Report.fail ("trying to free register "^(string_of_register reg)^" which is already free")
      else
	free := reg::(!free)
    else
      Report.fail ("trying to free register "^(string_of_register reg)^" which is not freeable")
  in
  let used_registers () = 
    List.filter (fun reg -> not (List.mem reg (!free))) all_free_registers
  in
    get_register,get_free_register,free_register,used_registers

type label = { label : string; mutable anchor : int option }

let string_of_label l = l.label

let new_label name = { label = name; anchor = None }

let isAnchored label =
  match label.anchor with
      None -> false
    | _ -> true

let getAnchor label = 
  match label.anchor with
      None -> Report.fail ("label "^(string_of_label label)^" still not anchored")
    | Some(n) -> n

let setAnchor label pc =
  if isAnchored label then
    Report.fail ("trying to anchor label "^(string_of_label label)^" which is already anchored")
  else
    label.anchor <- Some(pc)

let code () =
  let len = ref 1 in
  let pc = ref 0 and code = ref (Array.make !len (None,[],"")) in
  let emit i =
    let _,labels,com = (!code).(!pc) in
    (!code).(!pc) <- Some(i),labels,com;
    incr pc;
    if !pc >= !len then
      begin
	let a = Array.make (2 * !len) (None,[],"") in
	  Array.blit !code 0 a 0 !len;
	  code := a;
	  len := 2 * !len;
      end
  and comment com =
    let i,labels,ocom = (!code).(!pc) in
      if ocom = "" then
	(!code).(!pc) <- i,labels,com
  and getPC () = !pc 
  and setPC label = 
    setAnchor label !pc;
    let i,labels,com = (!code).(!pc) in
      (!code).(!pc) <- i,(label::labels),com
  and dump () =
    for i = 0 to (!pc - 1) do
      let Some(instr),labels,com = (!code).(i) in
	begin
	  match labels with
	      [] -> ()
	    | _::_ ->
		List.iter (function l -> 
			     print_string "/* ";
			     print_string (string_of_label l);
			     print_string " */";
			     print_newline()) (List.rev labels);
	end;
	print_string (string_of_instruction instr);
	if com <> "" then 
	  begin
	    print_string "\t/* ";
	    print_string com;
	    print_string " */"
	  end;
	print_newline()
    done
  in
    getPC,setPC,emit,comment,dump

