type token = 
    IDENT of string 
  | NUMBER of Int32.t
  | STRING of string
  | LPAREN
  | RPAREN
  | COLON
  | SEMICOLON
  | COMMA
  | CLASS
  | LBRACE
  | RBRACE
  | VAL
  | INT
  | ARROW
  | NOT
  | MOD
  | AND
  | OR
  | MUL
  | ADD
  | SUB
  | DOT
  | DIV
  | LT
  | LE
  | EQUAL
  | EQ
  | NEQ
  | GT
  | GE
  | DO
  | ELSE
  | FALSE
  | IF
  | LET
  | NEW
  | NULL
  | TRUE
  | VAR
  | WHILE
  | DEF
  | EOF
  | BAD

let token_representation = function
    IDENT(s) -> "IDENT<"^s^">"
  | NUMBER(n) -> "NUMBER<"^(Int32.to_string n)^">"
  | STRING(s) -> "STRING<"^s^">"
  | LPAREN -> "LPAREN"
  | RPAREN -> "RPAREN"
  | COLON -> "COLON"
  | SEMICOLON -> "SEMICOLON"
  | COMMA -> "COMMA"
  | CLASS -> "CLASS"
  | LBRACE -> "LBRACE"
  | RBRACE -> "RBRACE"
  | VAL -> "VAL"
  | INT -> "INT"
  | ARROW -> "ARROW"
  | NOT -> "NOT"
  | MOD -> "MOD"
  | AND -> "AND"
  | OR -> "OR"
  | MUL -> "MUL"
  | ADD -> "ADD"
  | SUB -> "SUB"
  | DOT -> "DOT"
  | DIV -> "DIV"
  | LT -> "LT"
  | LE -> "LE"
  | EQUAL -> "EQUAL"
  | EQ -> "EQ"
  | NEQ -> "NEQ"
  | GT -> "GT"
  | GE -> "GE"
  | DO -> "DO"
  | ELSE -> "ELSE"
  | FALSE -> "FALSE"
  | IF -> "IF"
  | LET -> "LET"
  | NEW -> "NEW"
  | NULL -> "NULL"
  | TRUE -> "TRUE"
  | VAR -> "VAR"
  | WHILE -> "WHILE"
  | DEF -> "DEF"
  | EOF -> "EOF"
  | BAD -> "BAD"

let string_of_token = function
    IDENT(s) -> s
  | NUMBER(n) -> Int32.to_string n
  | STRING(s) -> let g = String.make 1 (Char.chr 34) in g^s^g
  | LPAREN -> "("
  | RPAREN -> ")"
  | COLON -> ":"
  | SEMICOLON -> ";"
  | COMMA -> ","
  | CLASS -> "class"
  | LBRACE -> "{"
  | RBRACE -> "}"
  | VAL -> "val"
  | INT -> "Int"
  | ARROW -> "=>"
  | NOT -> "!"
  | MOD -> "%"
  | AND -> "&"
  | OR -> "|"
  | MUL -> "*"
  | ADD -> "+"
  | SUB -> "-"
  | DOT -> "."
  | DIV -> "/"
  | LT -> "<"
  | LE -> "<="
  | EQUAL -> "="
  | EQ -> "=="
  | NEQ -> "!="
  | GT -> ">"
  | GE -> ">="
  | DO -> "do"
  | ELSE -> "else"
  | FALSE -> "false"
  | IF -> "if"
  | LET -> "let"
  | NEW -> "new"
  | NULL -> "null"
  | TRUE -> "true"
  | VAR -> "var"
  | WHILE -> "while"
  | DEF -> "def"
  | EOF -> "<eof>"
  | BAD -> "<bad>"
